﻿--
-- $Id: BugGrabber.lua 6441 2006-08-01 09:38:54Z fritti $
--

MAX_BUGGRABBER_ERRORS = 1000

-- Localization
local USAGE = "Usage: /buggrabber <1-%d>."
local ERROR_INDEX = "The provided index must be a number."
local ERROR_UNKNOWN_INDEX = "The index %d does not exist in the load error table."
local STARTUP_ERRORS = "There were %d startup errors:"
local STARTUP_ERRORS_MANY = "There were %d startup errors, please use /buggrabber <number> to print them."
local UNIQUE_CAPTURE = "BugGrabber captured a unique error:\n%s\n---"
local ADDON_CALL_PROTECTED = "[%s] AddOn '%s' tried to call the protected function '%s'."
local ADDON_CALL_PROTECTED_MATCH = "^%[(.*)%] (AddOn '.*' tried to call the protected function '.*'.)$"

if GetLocale() == "koKR" then
	USAGE = "사용법: /buggrabber <1-%d>."
	ERROR_INDEX = "색인값은 숫자이어야 합니다."
	ERROR_UNKNOWN_INDEX = "불러온 오류목록에 %d번째 오류는 존재하지 않습니다."
	STARTUP_ERRORS = "%d개의 시작시 오류가 발생:"
	STARTUP_ERRORS_MANY = "%d개의 시작시 오류가 발생했습니다, /buggrabber <숫자>를 사용해서 해당 오류를 볼 수 있습니다."
	UNIQUE_CAPTURE = "벌레잡이가 발견한 오류:\n%s\n---"
elseif GetLocale() == "deDE" then
	USAGE = "Benutzung: /buggrabber <1-%d>."
	ERROR_INDEX = "Der zur Verfügung gestellte Index muß eine Zahl sein."
	ERROR_UNKNOWN_INDEX = "Der Index %d existiert nicht in der geladenen Fehlerliste."
	STARTUP_ERRORS = "Es gab %d Errors beim starten:"
	STARTUP_ERRORS_MANY = "Es gab %d Errors beim starten, verwende bitte /buggrabber <nummer>, um sie zu aufzulisten."
	UNIQUE_CAPTURE = "BugGrabber hat einen einzigartigen error aufgezeichnet:\n%s\n---"
elseif GetLocale() == "esES" then
	USAGE = "Uso: /buggrabber <1-%d>."
	ERROR_INDEX = "El \195\173ndice introducido debe ser un n\195\186mero."
	ERROR_UNKNOWN_INDEX = "El \195\173ndice %d no existe en la tabla de errores de carga."
	STARTUP_ERRORS = "Ha habido %d errores de arranque:"
	STARTUP_ERRORS_MANY = "Ha habido %d errores de arranque, por favor, usa /buggrabber <number> para listarlos."
	UNIQUE_CAPTURE = "BugGrabber ha capturado un \195\186nico error:\n%s\n---"
	ADDON_CALL_PROTECTED = "[%s] El accesorio '%s' ha intentado llamar a la funci\195\179n protegida '%s'."
	ADDON_CALL_PROTECTED_MATCH = "^%[(.*)%] (El accesorio '.*' ha intentado llamar a la funci\195\179n protegida '.*'.)$"
end

-- Create our event registering frame
local f = CreateFrame("Frame", "BugGrabber")

local slashCmdCreated = nil
local temporaryList = nil

-- State variables
BugGrabber.loaded = false
BugGrabber.loadErrors = {}
BugGrabber.bugsackErrors = {}
BugGrabber.errors = {}

-- Our persistent error database
BugGrabberDB = {}
BugGrabberDB.session = 0
BugGrabberDB.save = true
BugGrabberDB.limit = math.ceil(MAX_BUGGRABBER_ERRORS / 2)
BugGrabberDB.errors = {}

-- Determine the proper DB and return it
function BugGrabber.GetDB()
	if not BugGrabber.loaded then
		return BugGrabber.loadErrors
	end
	if not BugGrabberDB.save then
		return BugGrabber.errors
	end
	return BugGrabberDB.errors
end

-- Error handler
function BugGrabber.GrabError(err)
	-- Get the full backtrace
	local real = err:find("^.-([^\\]+\\)([^\\]-)(:%d+):(.*)$") or err:find("^%[string \".-([^\\]+\\)([^\\]-)\"%](:%d+):(.*)$") or err:find("^%[string (\".-\")%](:%d+):(.*)$") or err:find("^%[C%]:(.*)$")
	err = err .. "\n" .. debugstack(real and 4 or 3)
	local errorType = "error"

	-- Normalize the full paths into last directory component and filename.
	local errmsg = ""
	local looping = false
	for trace in err:gmatch("(.-)\n") do
		local match, found, path, file, line, msg
		found = false

		-- First detect an endless loop so as to abort it below
		if trace:find("BugGrabber") then
			looping = true
		end

		-- "path\to\file.lua:linenum:message"
		if not found then
			match, _, path, file, line, msg = trace:find("^.-([^\\]+\\)([^\\]-)(:%d+):(.*)$")
			if match then
				found = true
			end
		end

		-- "[string \"path\\to\\file.lua:<foo>\":linenum:message"
		if not found then
			match, _, path, file, line, msg = trace:find("^%[string \".-([^\\]+\\)([^\\]-)\"%](:%d+):(.*)$")
			if match then
				found = true
			end
		end

		-- "[string \"FOO\":linenum:message"
		if not found then
			match, _, file, line, msg = trace:find("^%[string (\".-\")%](:%d+):(.*)$")
			if match then
				found = true
				path = '<string>:'
			end
		end

		-- "[C]:message"
		if not found then
			match, _, msg = trace:find("^%[C%]:(.*)$")
			if match then
				found = true
				path = '<in C code>'
				file = ''
				line = ''
			end
		end

		-- ADDON_ACTION_BLOCKED
		if not found then
			match, _, file, msg = trace:find(ADDON_CALL_PROTECTED_MATCH)
			if match then
				found = true
				path = '<event>'
				file = 'ADDON_ACTION_BLOCKED'
				line = ''
				errorType = "event"
			end
		end

		-- Anything else
		if not found then
			path = trace--'<unknown>'
			file = ''
			line = ''
			msg = line
		end

		-- Add it to the formatted error
		errmsg = errmsg .. path .. file .. line .. ":" .. msg .. "\n"
	end

	-- Now store the error
	BugGrabber.SaveError(errmsg, errorType)
end

function BugGrabber.SaveError(message, errorType)
	-- Start with the date, time and session
	local oe = {}
	oe.message = message .. "\n  ---"
	oe.session = BugGrabberDB.session
	oe.time = date("%Y/%m/%d %H:%M:%S")
	oe.type = errorType
	oe.counter = 1

	-- WoW crashes when strings > 983 characters are stored in the
	-- SavedVariables file, so make sure we don't exceed that limit.
	-- For lack of a better thing to do, just truncate the error :-/
	if oe.message:len() > 980 then
		local m = oe.message
		oe.message = {}
		local maxChunks, chunks = 5, 0
		while m:len() > 980 and chunks <= maxChunks do
			local q
			q, m = m:sub(1, 980), m:sub(981)
			table.insert(oe.message, q)
			chunks = chunks + 1
		end
		if m:len() > 980 then m = m:sub(1, 980) end
		table.insert(oe.message, m)
	end

	local added = false

	-- Insert the error into the correct database if it's not there already.
	-- If it is, just increment the counter.
	local found = false
	local db = BugGrabber.GetDB()
	local oe_message = oe.message
	if type(oe_message) == "table" then
		oe_message = oe_message[1]
	end
	for i, err in ipairs(db) do
		local err_message = err.message
		if type(err_message) == "table" then
			err_message = err_message[1]
		end
		if err_message == oe_message and err.session == oe.session then
			-- This error already exists in the current session, just increment
			-- the counter on it.
			if not err.counter or type(err.counter) ~= "number" then
				err.counter = 1
			end
			err.counter = err.counter + 1

			-- Update the current error for the event firing later.
			oe.counter = err.counter
			oe.time = err.time
			found = true
			break
		end
	end

	-- If the error was not found in the current session, append it to the
	-- database.
	if not found then
		table.insert(db, oe)
		added = true
	end

	-- Also insert it into the temporary capture database that we maintain
	-- to silence loading errors while we wait for BugSack to load
	if BugGrabber.bugsackErrors and type(BugGrabber.bugsackErrors) == "table" then
		found = false
		local oe_message = oe.message
		if type(oe_message) == "table" then
			oe_message = oe_message[1]
		end
		for i, err in ipairs(BugGrabber.bugsackErrors) do
			local err_message = err.message
			if type(err_message) == "table" then
				err_message = err_message[1]
			end
			if err_message == oe_message and err.session == oe.session then
				if not err.counter or type(err.counter) ~= "number" then
					err.counter = 1
				end
				err.counter = err.counter + 1
				found = true
				break
			end
		end
		if not found then
			table.insert(BugGrabber.bugsackErrors, oe)
			added = true
		end
	end

	-- Save only the last <limit> errors (otherwise the SV gets too big)
	if #db > BugGrabberDB.limit then
		table.remove(db, 1)
	end

	-- Now trigger an event if someone's listening. If not, just print
	-- it to the chat frame so it doesn't get lost.
	if not looping and not BugGrabber.bugsackErrors then
		local event = AceLibrary and AceLibrary:HasInstance("AceEvent-2.0") and AceLibrary("AceEvent-2.0") or nil
		if event and event:IsEventRegistered("BugGrabber_BugGrabbed") then
			local prefix = "Bug"
			if errorType == "event" then prefix = "Event" end
			if added then
				event:TriggerEvent("BugGrabber_"..prefix.."Grabbed", oe)
			else
				event:TriggerEvent("BugGrabber_"..prefix.."GrabbedAgain", oe)
			end
		elseif added then
			BugGrabber.CreateSlashCmd()
			if not temporaryList then temporaryList = {} end
			table.insert(temporaryList, oe)
		end
	end
end

-- Event handlers
function BugGrabber.AddonLoaded()
	BugGrabber.loaded = true

	-- Persist defaults and make sure we have sane SavedVariables
	if not BugGrabberDB or type(BugGrabberDB) ~= "table" then
		BugGrabberDB = {}
	end
	if not BugGrabberDB.session or type(BugGrabberDB.session) ~= "number" then
		BugGrabberDB.session = 0
	end
	if not BugGrabberDB.errors or type(BugGrabberDB.errors) ~= "table" then
		BugGrabberDB.errors = {}
	end
	if not BugGrabberDB.limit or type(BugGrabberDB.limit) ~= "number" then
		BugGrabberDB.limit = 100
	end
	if BugGrabberDB.save == nil or type(BugGrabberDB.save) ~= "boolean" then
		BugGrabberDB.save = true
	end

	-- From now on we can persist errors. Create a new session.
	BugGrabberDB.session = BugGrabberDB.session + 1

	-- Determine the correct database
	local db = BugGrabber.GetDB()

	-- Cut down on the nr of errors if it is over 100
	while #db + #BugGrabber.loadErrors > BugGrabberDB.limit do
		table.remove(db, 1)
	end

	-- Save the errors that occurred while our variables were loading
	-- in the correct database.
	for _,err in pairs(BugGrabber.loadErrors) do
		err.session = BugGrabberDB.session
		table.insert(db, err)
	end

	-- Now do away with the temporary database
	BugGrabber.loadErrors = nil
end

function BugGrabber.PrintLoadError(index)
	if not index or tostring(index) == "" then
		DEFAULT_CHAT_FRAME:AddMessage(USAGE:format(temporaryList and #temporaryList or 1))
		return
	end
	if not tonumber(index) then
		DEFAULT_CHAT_FRAME:AddMessage(ERROR_INDEX)
		return
	end
	index = tonumber(index)
	if not temporaryList or not temporaryList[index] then
		DEFAULT_CHAT_FRAME:AddMessage(ERROR_UNKNOWN_INDEX:format(index))
		return
	end
	local err = temporaryList[index]
	if not err or not type(err) == "table" or not err.message or (type(err.message) ~= "string" and type(err.message) ~= "table") then return end
	if BugSack and type(BugSack.FormatError) == "function" then
		DEFAULT_CHAT_FRAME:AddMessage(tostring(index) .. ". " .. BugSack:FormatError(err))
	else
		local m = err.message
		if type(m) == "table" then
			m = table.concat(m, '')
		end
		DEFAULT_CHAT_FRAME:AddMessage(tostring(index) .. ". " .. m)
	end
end

function BugGrabber.CreateSlashCmd()
	if slashCmdCreated then return end
	local name = "BUGGRABBERCMD"
	local counter = 0
	repeat
		name = "BUGGRABBERCMD"..tostring(counter)
		counter = counter + 1
	until not _G.SlashCmdList[name] and not _G["SLASH_"..name.."1"]
	_G.SlashCmdList[name] = BugGrabber.PrintLoadError
	_G["SLASH_"..name.."1"] = "/buggrabber"

	slashCmdCreated = true
end

function BugGrabber.PlayerLogin()
	-- On player login, check to see whether we had load time errors and
	-- display them in the chat frame if we can't find BugSack. We cheat
	-- by letting BugSack reset BugGrabber.bugsackErrors so we can just
	-- check that.
	local err = BugGrabber.bugsackErrors
	if err and type(err) == "table" and #err > 0 then
		local num = #err
		local _G = getfenv(0)
		if num > 4 and type(_G.SlashCmdList) == "table" and type(_G) == "table" then
			DEFAULT_CHAT_FRAME:AddMessage(STARTUP_ERRORS_MANY:format(num))
			if not temporaryList then temporaryList = {} end
			for _, v in ipairs(err) do
				table.insert(temporaryList, v)
			end
			BugGrabber.CreateSlashCmd()
		else
			DEFAULT_CHAT_FRAME:AddMessage(STARTUP_ERRORS:format(num))
			for k, e in ipairs(err) do
				if BugSack and type(BugSack.FormatError) == "function" then
					DEFAULT_CHAT_FRAME:AddMessage(tostring(k) .. ". " .. BugSack:FormatError(e))
				else
					local m = e.message
					if type(m) == "table" then
						m = table.concat(m, '')
					end
					DEFAULT_CHAT_FRAME:AddMessage(tostring(k) .. ". " .. m)
				end
			end
		end
	end

	-- No need to wait for BugSack to load anymore
	BugGrabber.bugsackErrors = nil
end

function BugGrabber.OnEvent()
	if event == "ADDON_LOADED" then
		if arg1 == "!BugGrabber" then
			BugGrabber.AddonLoaded()
		elseif arg1 == "!Swatter" or (type(SwatterData) == "table" and SwatterData.enabled) then
			DisableAddOn("!Swatter")
			SwatterData.enabled = nil
			if geterrorhandler() ~= BugGrabber.GrabError then
				BugGrabber.seterrorhandler(BugGrabber.GrabError)
			end
			if SlashCmdList["SWATTER"] then
				SlashCmdList["SWATTER"] = nil
			end
			SLASH_SWATTER1, SLASH_SWATTER2 = nil, nil
			for k, v in pairs(Swatter) do Swatter[k] = nil end
			for k, v in pairs(SwatterData) do SwatterData[k] = nil end
			Swatter = nil
			SwatterData = nil
		end
	elseif event == "PLAYER_LOGIN" then
		BugGrabber.PlayerLogin()
	elseif event == "ADDON_ACTION_BLOCKED" or event == "ADDON_ACTION_FORBIDDEN" then
		BugGrabber.GrabError(ADDON_CALL_PROTECTED:format(event, arg1, arg2))
	end
end

-- Simple setters/getters for settings, meant to be accessed by BugSack
function BugGrabber.GetSave()
	return BugGrabberDB.save
end

function BugGrabber.ToggleSave()
	BugGrabberDB.save = not BugGrabberDB.save
	if BugGrabberDB.save then
		BugGrabberDB.errors = BugGrabber.errors
		BugGrabber.errors = {}
	else
		BugGrabber.errors = BugGrabberDB.errors
		BugGrabberDB.errors = {}
	end
end

function BugGrabber.GetLimit()
	return BugGrabberDB.limit
end

function BugGrabber.SetLimit(l)
	if not l or type(l) ~= "number" or l < 10 or l > MAX_BUGGRABBER_ERRORS then
		return false
	end

	BugGrabberDB.limit = math.floor(l)

	local db = BugGrabber.GetDB()
	while #db > l do
		table.remove(db, 1)
	end
	return true
end

-- Now register for our needed events
f:RegisterEvent("ADDON_LOADED")
f:RegisterEvent("PLAYER_LOGIN")
f:RegisterEvent("ADDON_ACTION_BLOCKED")
f:RegisterEvent("ADDON_ACTION_FORBIDDEN")
f:SetScript("OnEvent", BugGrabber.OnEvent)

seterrorhandler(BugGrabber.GrabError)
BugGrabber.seterrorhandler = seterrorhandler
function seterrorhandler() --[[ noop ]] end

-- vim:set ts=4:
