
-- create the statusbar for the inflight timer
-- basically a copy and paste of oCB's framework, and it will use oCB's settings if available
function InFlight:CreateBar()
	local frame = CreateFrame("Frame", "InFlightBar", UIParent)
	if InFlightVars.x then
		self.db.profile.x = InFlightVars.x
		InFlightVars.x = nil
		self.db.profile.y = InFlightVars.y
		InFlightVars.y = nil
	end
	if not self.db.profile.x then
		frame:SetPoint("CENTER", UIParent, "CENTER", 0, 200)
	else
		frame:SetPoint("TOPLEFT", UIParent, "TOPLEFT", self.db.profile.x, self.db.profile.y)
	end
	frame:SetMovable(true)
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", function() if IsShiftKeyDown() then this:StartMoving() end end)
	frame:SetScript("OnDragStop", function() 
		this:StopMovingOrSizing() 
		local a,b,c,d,e = this:GetPoint()
		if a == "TOPLEFT" and c == "TOPLEFT" then
			InFlight.db.profile.x = floor(d + 0.5)
			InFlight.db.profile.y = floor(e + 0.5)
		end
	end)
	frame:SetScript("OnEnter", function()
		GameTooltip:SetOwner(this, "ANCHOR_RIGHT")
		GameTooltip:SetText("InFlight")
		GameTooltip:AddLine(" Shift left-click and drag to move.")
		GameTooltip:AddLine(" Right-click for menu.")
		GameTooltip:Show()
	end)
	frame:SetScript("OnLeave", function() GameTooltip:Hide() end)
	
	frame.bar = CreateFrame("StatusBar", nil, frame)
	frame.spark = frame.bar:CreateTexture(nil, "OVERLAY")
	frame.locText = frame.bar:CreateFontString(nil, "OVERLAY")
	frame.timeText = frame.bar:CreateFontString(nil, "OVERLAY")
	
	self:LoadOptions()
	self.dewdrop:Register(InFlightBar,
		'children', function()
				self.dewdrop:FeedAceOptionsTable(self.opts)
			end,
		'cursorX', true,
		'cursorY', true
	)
	self:UpdateLook()
end

function InFlight:UpdateLook()
	local db
	local f, _ = GameFontHighlightSmall:GetFont()
	local frame = InFlightBar
	
	-- if available, use your oCB settings to customize the flight bar
	if oCB and self.db.profile.ocb then
		if self.db.profile.fill == "Deplete" then
			db = oCB.db.profile["MirrorBar"]
		else
			db = oCB.db.profile["CastingBar"]
		end
	end
	if db then
		frame:SetWidth(db.width+9)
		frame:SetHeight(db.height+10)
		frame:SetBackdrop({
			bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
			edgeFile = oCB.Borders[db.edgeFile], edgeSize = 16,
			insets = {left = 5, right = 5, top = 5, bottom = 5},
		})
		if db.BorderColor then
			frame:SetBackdropBorderColor(db.BorderColor.r, db.BorderColor.g, db.BorderColor.b, 0.8)
		else
			frame:SetBackdropBorderColor(0, 0, 0, 0.75)
		end
		if db.BackColor then
			frame:SetBackdropColor(db.BackColor.r, db.BackColor.g, db.BackColor.b, 0.4)
		else
			frame:SetBackdropColor(0, 0, 0, 0.5)
		end
		
		frame.bar:ClearAllPoints()
		frame.bar:SetPoint("CENTER", frame, "CENTER", 0, 0)
		frame.bar:SetWidth(db.width)
		frame.bar:SetHeight(db.height)
		if oCB.Textures then
			frame.bar:SetStatusBarTexture(oCB.Textures[db.texture])
		elseif oCB.surface then
			frame.bar:SetStatusBarTexture(oCB.surface:Fetch(db.texture))
		else
			frame.bar:SetStatusBarTexture("Interface\\AddOns\\InFlight\\smooth")
		end
		
		frame.spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
		frame.spark:SetWidth(16)
		frame.spark:SetHeight(db.height*2.44)
		frame.spark:SetBlendMode("ADD")
		
	else
		frame:SetWidth(239)
		frame:SetHeight(22)
		frame:SetBackdrop({
			bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 16,
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16,
			insets = {left = 5, right = 5, top = 5, bottom = 5},
		})
		frame:SetBackdropBorderColor(0.8, 0.8, 0.8, 0.8)
		frame:SetBackdropColor(0.3, 0.3, 0.3, 0.4)
		
		frame.bar:ClearAllPoints()
		frame.bar:SetPoint("CENTER", frame, "CENTER", 0, 0)
		frame.bar:SetWidth(230)
		frame.bar:SetHeight(12)
		frame.bar:SetStatusBarTexture(self.surface:Fetch(self.db.profile.texture))
		
		frame.spark:SetTexture("Interface\\CastingBar\\UI-CastingBar-Spark")
		frame.spark:SetWidth(14)
		frame.spark:SetHeight(12*2.44)
		frame.spark:SetBlendMode("ADD")
	end
	
	frame.timeText:SetFont(f, 12)
	frame.timeText:ClearAllPoints()
	frame.locText:SetFont(f, 12)
	frame.locText:ClearAllPoints()
	
	if self.db.profile.inline then
		frame.timeText:SetJustifyH("RIGHT")
		frame.timeText:SetWidth(frame:GetWidth())
		frame.timeText:SetPoint("RIGHT", frame, "RIGHT", -6, 0)

		frame.locText:SetJustifyH("LEFT")
		frame.locText:SetJustifyV("CENTER")
		frame.locText:SetWidth(frame:GetWidth()*0.7)
		frame.locText:SetPoint("LEFT", frame, "LEFT", 6, 0)	
	else
		frame.timeText:SetJustifyH("CENTER")
		frame.timeText:SetWidth(frame:GetWidth())
		frame.timeText:SetPoint("CENTER", frame, "CENTER", 0, 0)

		frame.locText:SetJustifyH("CENTER")
		frame.locText:SetWidth(max(200, frame:GetWidth()*1.2))
		frame.locText:SetPoint("BOTTOM", frame, "TOP", 0, 0)
	end	
end


function InFlight:LoadOptions()
	if self.optionsloaded then return end
	local db = self.db.profile
	self.opts = {
		type = "group",
		args = {
			header = {
				type = "header",
				name = "InFlight",
				order = 1,
			},
			frame = {
				type = "group",
				name = "Appearance",
				desc = "Change the way the statusbar look and behave.",
				order = 2,
				args = {
					oCB = {
						type = "toggle",
						order = 1,
						name = "Use oCB settings",
						desc = "Use the settings of oCB if available (will override local settings).",
						set = function(v) db.ocb = v self:UpdateLook() end,
						get = function() return db.ocb end,
					},
					inline = {
						type = "toggle",
						order = 2,
						name = "Inline mode",
						desc = "Shows only destination/time inside the statusbar",
						set = function(v) db.inline = v self:UpdateLook() 
							if not self.destination then return end
							if db.inline then
								InFlightBar.locText:SetText(self.destination)
							else
								InFlightBar.locText:SetText(self.source .. " --> " .. self.destination)
							end
						end,
						get = function() return db.inline end,
					},
					barfill = {
						type = "text",
						order = 3,
						name = "Bar fill type",
						desc = "Set the fill type of the statusbar.",
						set = function(v) db.fill = v self:UpdateLook() end,
						get = function() return db.fill end,
						validate = { "Fill up", "Deplete", },
					},
					barcolor = {
						type = "color",
						order = 4,
						name = "Bar color",
						desc = "Set the color of the fill bar.",
						get = function()
							local v = db.color
							return v.r,v.g,v.b
						end,
						set = function(r,g,b) 
							db.color = {r=r, g=g, b=b} 
							if not db.ocb then InFlightBar.bar:SetStatusBarColor(r, g, b) end
						end,
					},
					bartexture = {
						type = "text",
						order = 5, 
						name = "Bar Texture",
						desc = "Set the texture of the statusbar",
						get = function() return self.db.profile.texture end,
						set = function(v) 
							self.db.profile.texture = v
							if not db.ocb then InFlightBar.bar:SetStatusBarTexture(self.surface:Fetch(self.db.profile.texture)) end
						end,
						validate = self.surface:List(),
					},
				},
			},
			dismount = {
				type = "toggle",
				order = 4,
				name = "Auto-dismount",
				desc = "Auto-dismount when talking to a flightmaster.",
				set = function(v) db.dismount = v end,
				get = function() return db.dismount end,
			},
			confirm = {
				type = "toggle",
				order = 5,
				name = "Confirm flight",
				desc = "Show a confirmation window to check if you really want to take this flight.",
				set = function(v) db.confirmflight = v end,
				get = function() return db.confirmflight end,
			},
		},
	}
	self.dewdrop = AceLibrary("Dewdrop-2.0")
	self.optionsloaded = true
	self:RegisterEvent("Surface_Registered", function()
		self.opts.args.frame.args.bartexture.validate = self.surface:List()
	end)
end

