
InFlight = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceHook-2.1", "AceDB-2.0", "AceConsole-2.0", "Metrognome-2.0")
local L = AceLibrary("AceLocale-2.2"):new("InFlight")
InFlight.surface = AceLibrary("Surface-1.0")

L:RegisterTranslations("enUS", function() return {
	["Nighthaven"] = true,				--Nighthaven, Moonglade
	["I'd like to fly to (.+)."] = true,	--Druid gossip option
	["Plaguewood"] = true,				--Plaguewood, Eastern Plaguelands
	["Take me to (.+)."] = true,			--EPL Plaguewood Tower flight
} end)

function InFlight:OnInitialize()
	self:RegisterEvent("TAXIMAP_OPENED")
	self:RegisterDB("InFlightDB")
	self:RegisterDefaults("profile", {
		confirmflight = false,
		dismount = true, 
		ocb = true,
		fill = "Fill up",
		inline = false,
		color = { r = 0.2, g = 0.2, b = 0.2, },
		texture = "smooth",
	} )
	self:RegisterChatCommand({ "/inflight"}, {
		type = "execute", name = "InFlight", desc = "InFlight",
		func = function() 
			self:LoadOptions() 
			self.dewdrop:Open( UIParent, 'children', function() self.dewdrop:FeedAceOptionsTable(self.opts) end, 'cursorX', true, 'cursorY', true ) 
		end,
	})
	self.surface:Register("Smooth", "Interface\\AddOns\\InFlight\\smooth")
	
	if Cartographer_Notes then
		-- because sometimes faction group isn't initialized early after login
		self:RegisterEvent("AceEvent_FullyInitialized", "RegisterCarto")
	end
	
	self:Hook("GossipTitleButton_OnClick", "TakeMiscFlight", true)
end

function InFlight:RegisterCarto()
	if not self.db.profile.dataLoaded then InFlight:LoadPOIData() self.db.profile.dataLoaded = 1 end
	if not self.db.profile.removeOld then self:RemoveOldData() self.db.profile.removeOld = 1 end
	if not self.db.profile.poidata[UnitFactionGroup("player")] then self.db.profile.poidata[UnitFactionGroup("player")] = { } end
	Cartographer_Notes:RegisterIcon("TaxiIcon", { text = "Flight Master", path = "Interface\\TaxiFrame\\UI-Taxi-Icon-Green", })
	Cartographer_Notes:RegisterNotesDatabase("InFlight", self.db.profile.poidata[UnitFactionGroup("player")], self)
	Cartographer_Notes:RefreshMap()
end

-- setup only when the taxi map is opened the first time
function InFlight:TAXIMAP_OPENED(misc)
	if not self.initbulk then
		self.faction = UnitFactionGroup("player")
		self:Hook("TakeTaxiNode", true)
		self:SecureHook("TaxiNodeOnButtonEnter")
		
		-- function hooks to detect if a user took a port
		self:SecureHook("AcceptBattlefieldPort", function(index,accept) if accept then self.porttaken = 1 end end)
		self:SecureHook("ConfirmSummon", function() self.porttaken = 1 end)
		
		self:RegisterMetro(self.name, self.FlightTime, 0.25, self)
		if not InFlightVars then
			InFlightVars = { }
			self:LoadDefaults()
		end
		self.initbulk = 1
	end
	if not misc then
		for i = 1, NumTaxiNodes(), 1 do
			if TaxiNodeGetType(i) == "CURRENT" then
				self.source = self:ShortenName(TaxiNodeName(i))
				break
			end
		end
	end
	if self.db.profile.dismount then self:Dismount() end
	if Cartographer_Notes then self:NoteIt() end
end

-- add time info on the taxi map tooltips
function InFlight:TaxiNodeOnButtonEnter(button)
	
	local index = button:GetID()
	if TaxiNodeGetType(index) == "REACHABLE" then
		local destination = self:ShortenName(TaxiNodeName(index))

		if self.source and InFlightVars[self.faction][self.source] and InFlightVars[self.faction][self.source][destination] then
			local t = InFlightVars[self.faction][self.source][destination]
			if t == 0 then
				GameTooltip:AddLine("Time: -:--", 0.8, 0.8, 0.8)
			else
				GameTooltip:AddLine("Time: "..self:FormatTime(t), 1, 1, 1)
			end
		else
			GameTooltip:AddLine("Time: -:--", 0.8, 0.8, 0.8)
		end
		GameTooltip:Show()
	end
end

-- main hook to determine when a flight is taken
function InFlight:TakeTaxiNode(slot)
	if TaxiNodeGetType(slot) == "REACHABLE" then
		
		-- check to see if there's data for the source/destination pair
		self.destination = self:ShortenName(TaxiNodeName(slot))
		if InFlightVars[self.faction][self.source] and 
		   InFlightVars[self.faction][self.source][self.destination] and 
		   InFlightVars[self.faction][self.source][self.destination] ~= 0 then
			self.endTime = InFlightVars[self.faction][self.source][self.destination]
		else
			self.endTime = nil
		end

		-- confirm or start the flight
		if self.db.profile.confirmflight then
			self:ConfirmFlight(slot, self.destination, self.endTime)
		else
			self:StartTimer(slot)
		end
	end
end

-- setup the timer bar then begin timing
function InFlight:StartTimer(slot)
	self.totalTime = 0
	if not InFlightBar then self:CreateBar() end
	
	if self.db.profile.inline then
		InFlightBar.locText:SetText(self.destination)
	else
		InFlightBar.locText:SetText(self.source .. " --> " .. self.destination)
	end
	
	-- start the timers and setup statusbar
	if self.endTime then
		InFlightBar.bar:SetMinMaxValues(0, self.endTime)
		if self.db.profile.fill == "Deplete" then
			InFlightBar.bar:SetValue(self.endTime)
		else
			InFlightBar.bar:SetValue(0)
		end
		if oCB and self.db.profile.ocb then
			local c
			if self.db.profile.fill == "Deplete" then
				c = oCB.db.profile.Mirror.BREATH
			else
				c = oCB.db.profile.Colors.Casting
			end
			InFlightBar.bar:SetStatusBarColor(c.r, c.g, c.b, 0.7)
		else
			InFlightBar.bar:SetStatusBarColor(self.db.profile.color.r, self.db.profile.color.g, self.db.profile.color.b, 0.7)
		end
		InFlightBar.spark:Show()
	else
		InFlightBar.bar:SetMinMaxValues(0,1)
		InFlightBar.bar:SetValue(1)
		if oCB and self.db.profile.ocb then
			if self.db.profile.fill == "Deplete" then
				c = oCB.db.profile.Mirror.BREATH
			else
				c = oCB.db.profile.Colors.Channel
			end
			InFlightBar.bar:SetStatusBarColor(c.r, c.g, c.b, 0.7)
		else
			InFlightBar.bar:SetStatusBarColor(0.1, 0.1, 1.0, 0.7)
		end
		InFlightBar.spark:Hide()
	end
	self:StartMetro(self.name)
	self.porttaken = nil

	if slot then self.hooks.TakeTaxiNode(slot) end
end

-- in flight onupdate function
function InFlight:FlightTime(difference)
	if not self.active and UnitOnTaxi("player") then self.active = 1 end
	if not self.active then return end
	self.totalTime = self.totalTime + difference
	InFlightBar:Show()
	InFlightBar.bar:Show()
	if not UnitOnTaxi("player") then
		if not self.porttaken then self:SaveFlightTime() end
		InFlightBar:Hide()
		self.porttaken = nil
		self.endTime = nil
		self.active = nil
		self:StopMetro(self.name)
		return
	end
	if self.endTime then
		if self.totalTime - 2 > self.endTime  then
			InFlightBar.bar:SetMinMaxValues(0,1)
			InFlightBar.bar:SetValue(1)
			if oCB then
				local c = oCB.db.profile.Colors.Channel
				InFlightBar.bar:SetStatusBarColor(c.r, c.g, c.b, 0.7)
			else
				InFlightBar.bar:SetStatusBarColor(0.1, 0.1, 1.0, 0.7)
			end
			InFlightBar.spark:Hide()
			self.endTime = nil
		else
			if self.db.profile.fill == "Deplete" then
				InFlightBar.bar:SetValue(self.endTime - self.totalTime)
				InFlightBar.spark:SetPoint("CENTER", InFlightBar.bar, "LEFT", ((self.endTime - self.totalTime) / self.endTime) * InFlightBar.bar:GetWidth(), 0)
				InFlightBar.timeText:SetText(string.format("%s / %s", self:FormatTime(self.endTime - self.totalTime), self:FormatTime(self.endTime)))
			else
				InFlightBar.bar:SetValue(self.totalTime)
				InFlightBar.spark:SetPoint("CENTER", InFlightBar.bar, "LEFT", (self.totalTime / self.endTime) * InFlightBar.bar:GetWidth(), 0)
				InFlightBar.timeText:SetText(string.format("%s / %s", self:FormatTime(self.totalTime), self:FormatTime(self.endTime)))
			end
		end
	else
		InFlightBar.timeText:SetText("Timing - " .. self:FormatTime(self.totalTime))
	end
end

-- ask user to confirm flight before taking it
function InFlight:ConfirmFlight(slot, location, flighttime)
	local texttime = ""
	if flighttime then
		texttime = "This flight will take "..self:FormatTime(flighttime).."."
	end
	StaticPopupDialogs["INFLIGHT_CONFIRMTAKEOFF"] = {
		text = TEXT(string.format("Are you sure you want to fly to %s? %s", location, texttime)),
		button1 = TEXT(OKAY),
		button2 = TEXT(CANCEL),
		OnAccept = function(data)
			InFlight:StartTimer(data)
		end,
		timeout = 0,
		exclusive = 1,
		hideOnEscape = 1,
	}
	local dialog = StaticPopup_Show("INFLIGHT_CONFIRMTAKEOFF")
	if dialog then dialog.data = slot end
end

-- save data
function InFlight:SaveFlightTime()
	if not InFlightVars[self.faction][self.source] then
		InFlightVars[self.faction][self.source] = { }
	end
	InFlightVars[self.faction][self.source][self.destination] = floor(self.totalTime + 0.5)
	self.totalTime = 0
end


-- shorten name to lighten saved vars
function InFlight:ShortenName(name)
	local comma = ", "
	local found = string.find(name, comma)
	if found then name = string.sub(name, 1, found - 1) end
	return name
end

-- simple time format
function InFlight:FormatTime(secs)
	return string.format("%d:%02d", secs / 60, math.fmod(secs, 60))
end

-- dismount function, from FlightMap
function InFlight:Dismount()
	if not self.gratuity then
		-- borrowed from SpecialEvents-Mount
		self.gratuity = AceLibrary("Gratuity-2.0")
		self.tooltipfind = GetLocale() == "deDE" and "Erh\195\182\ht Tempo um (.+)%%"
			or GetLocale() == "koKR" and "이동 속도 ([60|100]*)%%만큼 증가"
			or GetLocale() == "frFR" and "Augmente la vitesse de (.+)%%"
			or "Increases speed by (.+)%%"
	end
	for i = 0, 31, 1 do
		local id, isAura = GetPlayerBuff(i, "HELPFUL")
		if isAura then
			self.gratuity:SetPlayerBuff(id)
			local txt, speed = self.gratuity:GetLine(2)
			if txt then _, _, speed = string.find(txt, self.tooltipfind) end
			if speed and tonumber(speed) > 40 then
				CancelPlayerBuff(id)
				break
			end
		elseif not id or id < 0 then 
			break
		end
	end
end

-- record POI location for Cartographer_Notes (you make me work so hard...)
function InFlight:NoteIt()
	if not InFlightVars[self.faction][self.source] then
		InFlightVars[self.faction][self.source] = { }
	end
	SetMapToCurrentZone()
	local x,y = GetPlayerMapPosition("player")
	if not Cartographer_Notes:GetNearbyNote(Cartographer:GetCurrentEnglishZoneName(), x, y, 0.05, "InFlight") then
		--[[zone, x, y, icon, creator,
			'title', title, 'titleR', titleR, 'titleG', titleG, 'titleB', titleB,
			'info', info, 'infoR', infoR, 'infoG', infoG, 'infoB', infoB,
			'info2', info2, 'info2R', info2R, 'info2G', info2G, 'info2B', info2B,
			'manual', manual]]--
		Cartographer_Notes:SetNote(
			Cartographer:GetCurrentEnglishZoneName(), x, y, "Interface\\TaxiFrame\\UI-Taxi-Icon-Green", "InFlight",
			"title", "Flight Master", "titleR", 0, "titleG", 1, "titleB", 0 )
	end
end

-- support for druid-only paths and EPL tower paths
function InFlight:TakeMiscFlight()
	self.hooks.GossipTitleButton_OnClick()
	if this:GetText() then
		local subzone = GetMinimapZoneText()
		if subzone == L["Nighthaven"] and strfind(this:GetText(), L["I'd like to fly to (.+)."]) then
			self.source = subzone
			_,_,self.destination = strfind(this:GetText(), L["I'd like to fly to (.+)."])
		elseif subzone == L["Plaguewood"] and strfind(this:GetText(), L["Take me to (.+)."]) then
			self.source = subzone
			_,_,self.destination = strfind(this:GetText(), L["Take me to (.+)."])
		else
			return
		end
		self:TAXIMAP_OPENED(true)
		if InFlightVars[self.faction][self.source] and InFlightVars[self.faction][self.source][self.destination] then
			self.endTime = InFlightVars[self.faction][self.source][self.destination]
		else
			self.endTime = nil
		end
		self:StartTimer()
	end
end
