﻿
local me = { name = "pet"}
local mod = thismod
mod[me.name] = me

--[[
Pet.lua

Adds Hunter and Warlock pets to the raid threat list.

Credit to Ghost for doing most of the work for me, also BlackArrow0203 and Namsar.
]]

-- talents: imp voidwalker (in KTM already(
-- improved succubus = 2, 7). 10% per tick. soothing kiss,.

-- Register polled methods. e.g. <me.redospellranks> will be called every 10 seconds.
me.myonupdates = 
{
	updatespellranks = 1,
	updateincombat = 1,
}

me.onload = function()

	me.parserset = mod.regex.createparser(me.parserdata)
	
end

me.myevents = { "CHAT_MSG_SPELL_PET_DAMAGE", "CHAT_MSG_COMBAT_PET_HITS" }

me.onevent = function()
	
	-- Check for any matches to our format strings
	local output = mod.regex.parse(me.parserset, arg1, event)
	
	if output.hit == nil then
		return
	end
	
	if output.parser.identifier == "whitedamage" then
		me.reportattack(output.final[3], output.final[3])
		
	elseif (output.parser.identifier == "physicaldamage") or (output.parser.identifier == "magicdamage") then
		
		me.reportattack(output.final[4], output.final[4])
	
	elseif output.parser.identifier == "spell" then
		
		local abilityid = mod.string.unlocalise("spell", output.final[2])
		
		if me.data[abilityid] then
			
			me.reportspell(abilityid)
		end
	end
	
end

me.data = 
{
	torment =
	{
		45, 75, 125, 215, 300, 395,
	},
	suffering = 
	{
		150, 300, 450, 600,
	},
	growl =
	{
		50, 65, 110, 170, 240, 320, 415,
	},
	intimidation = 
	{
		580,
	},
	cower = 
	{
		-30, -55, -85, -125, -175, -225,
	},
	soothingkiss =
	{
		-45, -75, -127, -165
	},
}

me.reportspell = function(abilityid)
	
	local data = me.data[abilityid]
	local rank = me.spellranks[abilityid]
	
	local value = data[rank]
	
	-- imp voidwalker for suffering
	if abilityid == suffering then
		value = value * mod.my.mods.warlock.voidwalker
	
	elseif abilityid == soothingkiss then
		value = value * mod.my.mods.warlock.succubus
	end 
	
	me.reportattack(value, 0)

end

me.reportattack = function(threat, damage)
	
	-- get current
	me.petthreat = mod.table.raiddata[UnitName("pet")]
	
	if me.petthreat == nil then
		me.petthreat = 0
	end
	
	-- add
	me.petthreat = me.petthreat + threat
	
	-- set
	mod.table.updateplayerthreat(UnitName("pet"),  me.petthreat)	
	KLHTM_RequestRedraw("raid")
	
	-- set personal
	if mod.table.mydata[UnitName("pet")] == nil then
		mod.table.mydata[UnitName("pet")] = mod.table.newdatastruct()
	end
	
	mod.table.mydata[UnitName("pet")].threat = me.petthreat
	mod.table.mydata[UnitName("pet")].hits = mod.table.mydata[UnitName("pet")].hits + 1
	mod.table.mydata[UnitName("pet")].damage = mod.table.mydata[UnitName("pet")].damage + damage
	
end

me.spellranks = { }

--[[
me.updatespellranks()
	Finds the rank of your pet's special spells, putting the values in <me.spellranks>.
]]
me.updatespellranks = function()
	
	local index = 0
	local name, rankstring, rank, spellid
	local rankpattern = mod.string.get("misc", "spellrank") -- e.g. "Rank %d" in english.
		
	while true do
		index = index + 1
		name, rankstring = GetSpellName(index, "pet")
		
		if name == nil then
			break
		end
		
		-- get the internal spell ID
		spellid = mod.string.unlocalise("spell", name)

		if spellid and me.data[spellid] then
			rank = 0
			
			_, _, rank = string.find(rankstring, rankpattern)
			
			if rank then
				me.spellranks[spellid] = tonumber(rank)
			end
		end
	end
	
	-- also add intimidation manually (it's a hunter ability, not a pet ability)
	me.spellranks.intimidation = 1
	
end

me.petincombat = false

--[[
me.updateincombat()
Remove the pet's threat when it goes out of combat.
]]
me.updateincombat = function()
	
	if UnitExists("pet") == nil then
		me.petincombat = false
		return
	end
	
	local combatnow = UnitAffectingCombat("pet") or UnitAffectingCombat("player")
	
	if (combatnow == nil) and (me.petincombat == true) then
		mod.table.raiddata[UnitName("pet")] = nil
		KLHTM_RequestRedraw("raid")		
	end
	
	me.petincombat = (combatnow ~= nil)
	
end

-- Parser for pet abilities
me.parserdata =
{
	-- Warlock Pet Taunt Spell
	{"spell", "SPELLCASTGOOTHERTARGETTED", "CHAT_MSG_SPELL_PET_DAMAGE"}, -- "%s casts %s on %s."
	-- Hunter Pet Taunt Spell
	{"spell", "SIMPLECASTOTHEROTHER", "CHAT_MSG_SPELL_PET_DAMAGE"}, -- "%s casts %s on %s."

	{"whitedamage", "COMBATHITCRITOTHEROTHER", "CHAT_MSG_COMBAT_PET_HITS"}, -- "%s crits %s for %d."
	{"whitedamage", "COMBATHITOTHEROTHER", "CHAT_MSG_COMBAT_PET_HITS"}, -- "%s hits %s for %d."

	-- Warlock Pet Damage Spell
	{"magicdamage", "SPELLLOGCRITSCHOOLOTHEROTHER", "CHAT_MSG_SPELL_PET_DAMAGE"},	-- "%s's %s crits %s for %d %s damage."
	{"magicdamage", "SPELLLOGSCHOOLOTHEROTHER", "CHAT_MSG_SPELL_PET_DAMAGE"}, -- "%s's %s hits %s for %d %s damage."
	
	-- Hunter Pet Damage Spell
	{"physicaldamage", "SPELLLOGCRITOTHEROTHER", "CHAT_MSG_SPELL_PET_DAMAGE"}, -- "%s's %s crits %s for %d."
	{"physicaldamage", "SPELLLOGOTHEROTHER", "CHAT_MSG_SPELL_PET_DAMAGE"}, -- "%s's %s hits %s for %d."
}