﻿
BINDING_HEADER_MOUNTME = "MountMe"


----------------------------
--      Localization      --
----------------------------

local L = AceLibrary("AceLocale-2.2"):new("MountMe")

L:RegisterTranslations("enUS", function() return {
	now = true,
	["Mount Me Now!"] = true,
	["Mounts with the best available mount or travel form"] = true,
} end)


------------------------------
--      Are you local?      --
------------------------------

-- AceLibrary("SpecialEvents-Equipped-2.0") -- For InBed
local pt = AceLibrary("PeriodicTable-2.0")
local selearn = AceLibrary("SpecialEvents-LearnSpell-2.0")
local semove = AceLibrary("SpecialEvents-Movement-2.0")
local seaura = AceLibrary("SpecialEvents-Aura-2.0")
local semount = AceLibrary("SpecialEvents-Mount-2.0")
local tablet = AceLibrary("Tablet-2.0")
local dewdrop = AceLibrary("Dewdrop-2.0")
local crayon = AceLibrary("Crayon-2.0")
local BS = AceLibrary("Babble-Spell-2.2")

local _, myclass = UnitClass("player")
local spellmounts = {[BS["Summon Felsteed"]] = 60, [BS["Summon Warhorse"]] = 60, [BS["Summon Dreadsteed"]] = 100, [BS["Summon Charger"]] = 100}
local defaulticon = "Interface\\Icons\\Ability_Kick"
local buffs, boosts, mountused, mmitemswap = {}, {}
local itemslots = {carrot = 13, spurs = 8, gloves = 10}
local itemboosts = {carrot = 3, spurs = 8, gloves = 4}
local invslots = {[8] = true, [10] = true, [13] = true, [14] = true, [true] = true}
local buffspeeds = {
	[BS["Aspect of the Pack"]] = 30,
	[BS["Aspect of the Cheetah"]] = 30,
	[BS["Ghost Wolf"]] = 40,
	[BS["Travel Form"]] = 40,
	[BS["Aquatic Form"]] = 50,
}


-------------------------------------
--      Namespace Declaration      --
-------------------------------------

MountMe = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceModuleCore-2.0", "AceConsole-2.0", "AceDB-2.0", "AceDebug-2.0", "FuBarPlugin-2.0")
MountMe:RegisterDB("MountMeDB", "MountMeDBPerChar")
MountMe.cmdtable = {type = "group", handler = MountMe, args = {
--~ 	[L["now"]] = {
--~ 		type = "execute",
--~ 		name = L["Mount Me Now!"],
--~ 		desc = L["Mounts with the best available mount or travel form"],
--~ 		func = "Now",
--~ 		order = 1,
--~ 	},
}}
MountMe:RegisterChatCommand({"/mme", "/mountme"}, MountMe.cmdtable)
MountMe.hasIcon = defaulticon
MountMe.independentProfile = true
MountMe.OnMenuRequest = MountMe.cmdtable


---------------------------
--      Ace Methods      --
---------------------------

function MountMe:OnInitialize()
	for name,module in self:IterateModules() do
		if module.consoleOptions then
			self.cmdtable.args[module.consoleCmd] = module.consoleOptions
		end
	end
end


function MountMe:OnEnable()
	mmitemswap = self:HasModule("ItemSwap") and self:GetModule("ItemSwap")

	self:RegisterEvent("ZONE_CHANGED_NEW_AREA", SetMapToCurrentZone)
	self:RegisterEvent("SpecialEvents_Mounted")
	self:RegisterEvent("SpecialEvents_Dismounted")
	self:RegisterEvent("SpecialEvents_PlayerBuffGained")
	self:RegisterEvent("SpecialEvents_PlayerBuffLost")
	self:RegisterEvent("MountMe_Dismount")
	self:RegisterEvent("MountMe_Deshift")
	self:RegisterEvent("SpecialEvents_EquipmentChanged")
end


-----------------------------
--      FuBar Methods      --
-----------------------------

function MountMe:OnTextUpdate()
	local s = 0
	for _,val in pairs(buffs) do s = s + val end
	for _,val in pairs(boosts) do s = s + val end
	self:SetText(string.format("|cff%s%s%%|r", crayon:GetThresholdHexColor(s, 0, 60, 75, 100, 125), s))
end


function MountMe:OnTooltipUpdate()
	local cat = tablet:AddCategory("text", "Buffs", "columns", 2)
	for i,val in pairs(buffs) do cat:AddLine("text", i, "text2", string.format("+%d%%", val)) end

	local cat2 = tablet:AddCategory("text", "Boosts", "columns", 2)
	for i,val in pairs(boosts) do
		local link = GetInventoryItemLink("player", itemslots[i])
		if link then _, _, link = string.find(link, "(item:%d+:%d+:%d+:%d+)") end
		local name = link and GetItemInfo(link)
		cat2:AddLine("text", name, "text2", string.format("+%d%%", val))
	end
end


------------------------------
--      Event Handlers      --
------------------------------


function MountMe:SpecialEvents_Mounted(buff, speed, idx)
	buffs[buff] = speed
	self:SpecialEvents_EquipmentChanged(true)
	self:SetIcon(GetPlayerBuffTexture(GetPlayerBuff(idx, "HELPFUL")))
	self:Update()
end


function MountMe:SpecialEvents_Dismounted(buff)
	mountused = nil
	buffs[buff] = nil
	self:SpecialEvents_EquipmentChanged(true)
	self:SetIcon(defaulticon)
	self:Update()
end


function MountMe:SpecialEvents_PlayerBuffGained(buff, idx)
	if buffspeeds[buff] then
		buffs[buff] = buffspeeds[buff]
		self:SetIcon(GetPlayerBuffTexture(GetPlayerBuff(idx, "HELPFUL")))
		self:Update()
	end
end


function MountMe:SpecialEvents_PlayerBuffLost(buff)
	if buffs[buff] then self:SetIcon(defaulticon) end
	buffs[buff] = nil
	self:Update()
end


function MountMe:SpecialEvents_EquipmentChanged(i)
	if not mmitemswap then return end

	if invslots[i] then
		local mounted = semount:PlayerMounted()
		for item,val in pairs(itemslots) do
			boosts[item] = mounted and mmitemswap:CheckItem(item, val) and itemboosts[item]
		end
		if i ~= true then self:Update() end
	end
end


function MountMe:MountMe_Dismount()
	if not IsMounted() or UnitOnTaxi("player") then return end

	local mount = semount:PlayerMounted()
	local buff = GetPlayerBuffName(mount)

	if buff then
		CancelPlayerBuff(buff)
		return true
	end
end


function MountMe:MountMe_Deshift()
	if myclass == "SHAMAN" then
		local i = GetPlayerBuffName(BS["Ghost Wolf"])
		if i then
			CancelPlayerBuff(i)
			return true
		end
	end
end

