local L = AceLibrary("AceLocale-2.2"):new("KkbKFu")
local tablet = AceLibrary("Tablet-2.0")   
local dewdrop = AceLibrary("Dewdrop-2.0")

kkbk = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "AceDebug-2.0", "AceConsole-2.0", "AceDB-2.0", "FuBarPlugin-2.0")
kkbk.hasIcon = true
kkbk.clickableTooltip = true
kkbk.version = "0.9.3";
kkbk.category = "Raid";

-- allows make real test
local really_debug = false

-- supported classes
local classes = { "Druid", "Priest", "Mage", "Paladin" }

-- buff for those classes
local buffs = {
	["Druid"] 	= {	"MARKOFWILD", "THORNS" },
	["Priest"]	= { "FORTITUDE", "SPIRIT", "SHADOW" },
	["Mage"] 	= { "INTELLECT", "AMPLIFY", "DAMPEN" },
	["Paladin"] = { "KING", "WISDOM", "MIGHT", "SALVATION", "SANCTUARY", "LIGHT" },
}

local assign = {}
local options = {}
-- test if number of parties has changed
local oldNumParties = {}

-- register rosterlib for join/leave people
function kkbk:OnEnable()
	self:RegisterEvent("RosterLib_RosterChanged")	
end

local in_raid = false

function kkbk:RosterLib_RosterChanged(tbl)
	if GetNumRaidMembers() > 0 then
		if not in_raid then
			in_raid = true
			self:changeNumParties()
			self:requestAssignments()
			return
		end
		for n in pairs(tbl) do
			local u = tbl[n]
			if not u.name then
				if self:acceptClass(u.oldclass) then
					self:addRemoveFromRaid(u.oldname, u.oldclass, false)
				end
			elseif not u.oldname then
				if self:acceptClass(u.class) then
					self:addRemoveFromRaid(u.name, u.class, true)
				end
			elseif u.subgroup ~= u.oldsubgroup then
					self:changeNumParties()
			end
		end
	else
		if in_raid then
			self:Debug("leave raid")
			in_raid = false
			oldNumParties = {}
			self:reset()
		end
	end
end

function kkbk:acceptClass(class)
	for _, v in pairs(classes) do
		if v == class then
			return true
		end
	end
	return false
end

function kkbk:addRemoveFromRaid(name, class, join)
	self:Debug(name, class, join)
	if self:IsDebugging() and really_debug then
		class = self:getOwnClass()
	end
	
	self:Debug("addRemoveFromRaid", name, class, join)
	if class == "Priest" or class == "Druid" or class == "Mage" or class == "Paladin" then
		local need_to_reload = false
		
		if join then
			need_to_reload = true
			for _, v in pairs(buffs[class]) do
				-- add to assignments
				if class == "Paladin" then
					if assign[class] then
						if assign[class][v] then
							assign[class][v][name] = false
						end
					end
				else
					if assign[class] then
						if assign[class][v] then
							assign[class][v][name] = {}
							for _, v in pairs(self:getNumInRaid()) do
								assign[class][v][name][v] = false
							end

							--[[for o=1, self:getNumInRaid() do
								assign[class][v][name][o] = false
							end]]
						end
					end
				end
				
				-- add to menu
				local b = v
				if options["args"]["assignments"]["args"][class]["args"][b] then					
					if class == "Paladin" then
						options["args"]["assignments"]["args"][class]["args"][b]["args"][name] = {
							name = L[b],
							type = "toggle",
							desc = L[b],
							set = function(v)
								assign[class][b][name] = not assign[class][b][name]
								self:sendAddon(class, b, name, nil, assign[class][b][name], "Paladin")
							end,
							get = function()
								return assign[class][b][name]
							end
						}
					else
						options["args"]["assignments"]["args"][class]["args"][b]["args"][name] = {
							name = name,
							type = "group",
							desc = name,
							args = {}
						}
						
						for _, v in pairs(groups_in_raid) do
								local st = "group" .. v
								local n = v
								local d = name
								local c = class

								options["args"]["assignments"]["args"][c]["args"][b]["args"][d]["args"][st] = {
									name = L:GetTranslation("group") .. " " .. n,
									type = "toggle",
									desc = L:GetTranslation("group") .. " " .. n,
									set = function(v)
										assign[c][b][d][n] = not assign[c][b][d][n]	
										self:sendAddon(c, b, d, n, assign[c][b][d][n])
									end,
									get = function()
										return assign[c][b][d][n]
									end
								}
							end
						
						--[[for o=1, self:getNumInRaid() do
							local st = "group" .. o
							local n = o
							local d = name
							local c = class
							
							options["args"]["assignments"]["args"][c]["args"][b]["args"][d]["args"][st] = {
								name = L:GetTranslation("group") .. " " .. n,
								type = "toggle",
								desc = L:GetTranslation("group") .. " " .. n,
								set = function(v)
									assign[c][b][d][n] = not assign[c][b][d][n]	
									self:sendAddon(c, b, d, n, assign[c][b][d][n])
								end,
								get = function()
									return assign[c][b][d][n]
								end
							}
						end]]
					end
				end
			end						
		else
			-- remove from assignments
			if assign[class] then
				for _, v in pairs(buffs[class]) do
					
					if assign[class][v] then					
						if assign[class][v][name] ~= nil then
							assign[class][v][name] = nil
						end
					end
				end
			end
			
			-- remove from menu			
			if options["args"]["assignments"]["args"][class] then
				for _, v in pairs(buffs[class]) do
					v = string.gsub(v, " ", "_")
					if options["args"]["assignments"]["args"][class]["args"][v] then
						if options["args"]["assignments"]["args"][class]["args"][v]["args"][name] then
							options["args"]["assignments"]["args"][class]["args"][v]["args"][name] = nil
							need_to_reload = true
						end
					end
				end
			end
		end
		
		if need_to_reload then
			kkbk:RegisterChatCommand({"/kkbk"}, options)
			kkbk.OnMenuRequest = options
			--self:changeNumParties()
		end
	end
end

function kkbk:UpdateText()
	self:SetText("kkbk")
end

function kkbk:OnInitialize() 
	--self:SetDebugging(true)
	--self:SetDebugLevel(3)
	
	self.rosterlib = AceLibrary("RosterLib-2.0")
	
	self:RegisterDB("kkbkDB")
	self:RegisterDefaults("profile", {
	    channel = L:GetTranslation("Channel"),
	    channel_set = "",
	    show_only_my_class = true,
	    one_line = true,
	    buff_format = "[$u] $b => [$g]",
	    paladin_buff_format = "[$u] => $b",
	    separator = ", ",
	    answer_to_whisp = true,
	} )
	
	self:RegisterEvent("CHAT_MSG_WHISPER")
	self:RegisterEvent("CHAT_MSG_ADDON")
	
	options = {
		type = "group", 
		name = L:GetTranslation("Options"),
		handler = kkbk,
		args = {
			options = {
				name = L:GetTranslation("Options"),
				type = "group",
				desc = L:GetTranslation("Options"),
				args = {
					channel = {
						name = L:GetTranslation("Send to"),
						type = "text", 
						desc = L:GetTranslation("Send to"),
						get = "getChannel",
						set = "setChannel",
						validate = { L:GetTranslation("Channel"), L:GetTranslation("Raid"), L:GetTranslation("Party") },
						order = 1
					},
					channel_set = {		
						name = L:GetTranslation("Channel to send"),
						type = "text", 
						desc = L:GetTranslation("Channel to send"),
						usage =  L:GetTranslation("Channel to send"),
						get = function() return self.db.profile.channel_set end,
						set = function(v) self.db.profile.channel_set = v end,
						order = 2
					},
					show_only_my_class = {
						name = L:GetTranslation("Show only own class"),
						type = "toggle",
						desc = L:GetTranslation("Show only own class"),
						get = function() return self.db.profile.show_only_my_class end,
						set = function(v) 
							self.db.profile.show_only_my_class = not self.db.profile.show_only_my_class 
							
							self:reset()
							self:changeNumParties()
							self:UpdateText()
						end,
						order = 3
					},
					display_text = {
						name = L:GetTranslation("Sending options"),
						type = "group",
						desc = L:GetTranslation("Sending options"),
						args = {
							one_line = {
								name = L:GetTranslation("Send every player on one line"),
								type = "toggle", 
								desc = L:GetTranslation("Send every player on one line"),
								get = function() return self.db.profile.one_line end,
								set = function(v) self.db.profile.one_line = not self.db.profile.one_line end,
								order = 1
							},
							buff_format = {
								name = L:GetTranslation("Buff format"),
								type = "text", 
								desc = L:GetTranslation("Buff format"),
								usage =  L:GetTranslation("Buff format"),
								get = function() return self.db.profile.buff_format end,
								set = function(v) self.db.profile.buff_format = v end,
								order = 2
							},
							paladin_buff_format = {
								name = L:GetTranslation("Paladin buff format"),
								type = "text", 
								desc = L:GetTranslation("Paladin buff format"),
								usage =  L:GetTranslation("Paladin buff format"),
								get = function() return self.db.profile.paladin_buff_format end,
								set = function(v) self.db.profile.paladin_buff_format = v end,
								order = 3
							},
							separator = {
								name = L:GetTranslation("Party separator"),
								type = "text", 
								desc = L:GetTranslation("Party separator"),
								usage =  L:GetTranslation("Party separator"),
								get = function() return self.db.profile.separator end,
								set = function(v) self.db.profile.separator = v end,
								order = 3
							},
						},
						order = 4
					},
					answer_to_whisp = {
						name = L:GetTranslation("Answer to whisps ?"),
						type = "toggle", 
						desc = L:GetTranslation("Answer to whisps ?"),
						get = function() return self.db.profile.answer_to_whisp end,
						set = function(v) 
							self.db.profile.answer_to_whisp = not self.db.profile.answer_to_whisp 
							
							if self.db.profile.answer_to_whisp then
								self:RegisterEvent("CHAT_MSG_WHISPER")
							else
								self:UnregisterEvent("CHAT_MSG_WHISPER")
							end	
						end,
						order = 5
					},
					request_version = {
						name = L:GetTranslation("Request version"),
						type = "execute",
						desc = L:GetTranslation("Request version"),
						func = function()
							self:requestVersion()
						end,
						order = 6
					},
					request_buff = {
						name = L:GetTranslation("Request assignments"),
						type = "execute",
						desc = L:GetTranslation("Request assignments"),
						func = function()
							self:requestAssignments()
						end,
						order = 7
					},
					reset = {
						name = L:GetTranslation("Reset"),
						type = "execute",
						desc = L:GetTranslation("Reset"),
						func = function() 							
							self:reset()
							self:changeNumParties()
							self:UpdateText()
						end,
						order = 8
					},
				},
				order = 1
			},
			assignments = {
				type = "group",
				name = L:GetTranslation("Assignments"),
				desc = L:GetTranslation("Assignments"),
				args = {},
				order = 2
			}
		}
	}
	oldNumParties = self:getNumInRaid()
	self:changeNumParties()
end

-- reset kkbk
function kkbk:reset()	
	options["args"]["assignments"]["args"] = {}
	assign = {}
end

-- return number of raid members
function kkbk:getNumInRaid()
	--[[
	local players_in_raid = GetNumRaidMembers()
	
	if self:IsDebugging() and really_debug then
		players_in_raid = 20
	end
	return ceil(players_in_raid / 5)
	]]

	--[[if self:IsDebugging() and really_debug then
		arr = {}
		foo = {1, 1, 1, 2, 2, 8, 3, 4, 6, 6, 8, 2, 2, 3, 4 , 1, 2}
		for i=1, 10 do
			subgroup = foo[i]
			if not self:inArray(arr, subgroup) then
				table.insert(arr, subgroup)
			end
		end
		return arr
	else]]
		arr = {}
		for i=1, GetNumRaidMembers() do
			_, _, subgroup = GetRaidRosterInfo(i)
			if not self:inArray(arr, subgroup) then
				table.insert(arr, subgroup)
			end
		end
		return arr
	--end
end

function kkbk:inArray(arr, value)
	for _, v in pairs(arr) do
		if v == value then return true end
	end
	return false
end

function kkbk:changeNumParties()
	local a = self:getNumInRaid()
	self:buildAssignments()

	for _, v in pairs(oldNumParties) do
		local removed = true

		for _, v1 in pairs(a) do
			self:Debug("test", v, v1)
			if v == v1 then
				removed = false
			end
		end

		if removed then
			self:removeFromAssign(v)
		end
	end

	--[[local n = self:getNumInRaid()
	if n > oldNumParties then
		oldNumParties = n
		self:buildAssignments()
	elseif n < oldNumParties then
		oldNumParties = n
		
		for k, class in pairs(assign) do
			for kk, player in pairs(class) do
				for kkk, buff in pairs(player) do
					if k == "Paladin" then
					else
						for i=table.getn(buff), oldNumParties, -1 do
							-- TODO REMOVE group self:Debug(i)
							
						end
					end
				end
			end
		end		
	end
	]]
end

function kkbk:removeFromAssign(group)
	self:Debug("remove group", group)

	for k, class in pairs(assign) do
		for kk, player in pairs(class) do
			for kkk, buff in pairs(player) do
				if k == "Paladin" then
				else
					assign[k][kk][kkk][group] = false
				end
			end
		end
	end
end

function kkbk:getOwnClass()
	if self:IsDebugging() and really_debug then 
		englishClass = "Priest"
	else 
		_, englishClass = UnitClass("player")
		englishClass = string.sub(englishClass, 0, 1) .. string.lower(string.sub(englishClass, 2))
	end
	
	return englishClass
end

-- build menu
function kkbk:buildAssignments()
	local groups_in_raid = self:getNumInRaid()
		
	--if groups_in_raid > 0 then
	if table.getn(groups_in_raid) > 0 then
		in_raid = true
		playerClass = self:getOwnClass()
		
		for i=1, table.getn(classes) do
			local s = classes[i]
			if (self.db.profile.show_only_my_class and string.upper(s) == string.upper(playerClass)) 
							or not self.db.profile.show_only_my_class then
				local players = self:getClass(s)
				
				-- init array if need
				if not assign[s] then
					assign[s] = {}
				end
				--self:Debug(s)
				--local st = classes[i]
				options["args"]["assignments"]["args"][s] = {
					name = L[s], 
					desc = L[s],
					type = "group",
					args = {},
					order = i
				}
				
				for ii=1, table.getn(buffs[s]) do
					local p = buffs[s][ii]
					local pw = p
					
					-- init array if need
					if assign[s][p] == nil then
						assign[s][p] = {}
					end
					
					options["args"]["assignments"]["args"][s]["args"][pw] = {
						name = L[p][1],
						type = "group",
						desc = L[p][1],
						args = {}
					}
					  for iii=1, table.getn(players) do
					  	local d = players[iii]
					  	
						if string.upper(s) == string.upper("Paladin") then
							-- init array if need
							if assign[s][p][d] == nil then							
								assign[s][p][d] = false
							end
							options["args"]["assignments"]["args"][s]["args"][pw]["args"][d] = {
								name = d,
								type = "toggle",
								desc = d,
								set = function(v)
									assign[s][p][d] = not assign[s][p][d]
									self:sendAddon(s, p, d, nil, assign[s][p][d], "Paladin")
								end,
								get = function()
									return assign[s][p][d]
								end
							}
						else 
							-- init array if need
							if assign[s][p][d] == nil then								
								assign[s][p][d] = {}
							end										
							options["args"]["assignments"]["args"][s]["args"][pw]["args"][d] = {
								name = d,
								type = "group",
								desc = d,
								args = {}
							}
							
							for _, v in pairs(groups_in_raid) do
								local st = "group" .. v
								local n = v
								if assign[s][p][d][n] == nil then
									assign[s][p][d][n] = false
								end

								options["args"]["assignments"]["args"][s]["args"][pw]["args"][d]["args"][st] = {
									name = L:GetTranslation("group") .. " " .. n,
									type = "toggle",
									desc = L:GetTranslation("group") .. " " .. n,
									set = function(v)
										assign[s][p][d][n] = not assign[s][p][d][n]
										self:sendAddon(s, p, d, n, assign[s][p][d][n])
									end,
									get = function()
										return assign[s][p][d][n]
									end
								}
							end


							--[[for o=1, groups_in_raid do
								local st = "group" .. o
								local n = o
								assign[s][p][d][o] = false
								
								options["args"]["assignments"]["args"][s]["args"][pw]["args"][d]["args"][st] = {
									name = L:GetTranslation("group") .. " " .. n,
									type = "toggle",
									desc = L:GetTranslation("group") .. " " .. n,
									set = function(v)
										assign[s][p][d][n] = not assign[s][p][d][n]
										self:sendAddon(s, p, d, n, assign[s][p][d][n])
									end,
									get = function()
										return assign[s][p][d][n]
									end
								}
							end]]
						end
					end
				end
			end
		end
	end
	kkbk:RegisterChatCommand({"/kkbk"}, options)
	kkbk.OnMenuRequest = options
end

function kkbk:printTable()
	for k, class in pairs(assign) do
		for kk, player in pairs(class) do
			for kkk, buff in pairs(player) do
				if k == "Paladin" then
					local s = ""
					if buff then s = "true" else s = "false" end
					self:Debug(k .. "." .. kk .. ".".. kkk .. "." .. " = " .. s)					
				else
					for kkkk, groups in pairs(buff) do
						local s = ""
						if groups then s = "true" else s = "false" end
						self:Debug(k .. "." .. kk .. ".".. kkk .. "." .. kkkk .. " = " .. s)
					end
				end
			end
		end
	end
end

function kkbk:getClass(name)
	local t = {}
	if self:IsDebugging() and really_debug then
        -- for debug mode, I use people from my own guild
        if name == "Paladin" then
        	t = { "Gladiateur", "Coinette", "Halgrim", "Ulrich"}
        elseif name == "Priest" then
            t = { "Lateria", "Elyxsir", "Pristina", "Khardinal", "Frereantho"}
        elseif name == "Mage" then
            t = { "Kiliana", "Opi", "Tarmani", "Vitac"}
        elseif name == "Druid" then
            t = { "Belldand", "Thegrimreaper", "Piqueplume"}
        end
    else
		local o=1
	    for i=1, GetNumRaidMembers() do
	        na = GetRaidRosterInfo(i)
	        if na ~= nil then
	            playerClass, englishClass = UnitClass("raid"..i);
	            if englishClass ~= nil and playerClass ~= nil then
	            	self:Debug(string.upper(englishClass), string.upper(name))
	                if string.upper(englishClass) == string.upper(name) then
	                    t[o] = na
	                    self:Debug("add "..na.." to " .. name)
			            o = o + 1
	                end
	            end
	        end
	    end
	end
	return t
end

function kkbk:setChannel(channel)
	if channel ~= L:GetTranslation("Raid") and channel ~= L:GetTranslation("Party") and channel ~= L:GetTranslation("Channel")  then
		channel = L:GetTranslation("Channel")
	end
	self.db.profile.channel = channel
	self:UpdateText()
end

function kkbk:getChannel()
	return self.db.profile.channel
end

function kkbk:OnTooltipUpdate()
    local cat = tablet:AddCategory(
        'text', L:GetTranslation("Options"),
        'columns', 2,
        'child_textR', 1,
        'child_textG', 1,
        'child_textB', 0,
        'child_textR2', 1,
        'child_textG2', 1,
        'child_textB2', 1
    )
    cat:AddLine(
		'text', L:GetTranslation("Send to"),
		'text2', self.db.profile.channel
	)
	if self.db.profile.channel == L:GetTranslation("Channel") then
		cat:AddLine(
			'text', L:GetTranslation("Channel"),
			'text2', self.db.profile.channel_set
		)
	end
	
    cat:AddLine()
    
    cat = tablet:AddCategory(
        'text', "Buffs",
        'columns', 1,
        'child_textR', 1,
        'child_textG', 1,
        'child_textB', 0,
        'child_textR2', 1,
        'child_textG2', 1,
        'child_textB2', 1
    )
    for k, class in pairs(assign) do
    	self:add(k, class)
	end
    
    tablet:SetHint(L:GetTranslation("Right-click for options, Shift-click to send assignments"))
end

function kkbk:add(name, class)
	local showClass = false
	local t = {}
	
	for k, buff in pairs(class) do
		local showBuff = false
		
		for kk, player in pairs(buff) do
			local showPlayer = false
			local i = 1
			
			if string.upper(name) == string.upper("Paladin") then
				if player then
					showClass = true
					if not t[k] then
						t[k] = {}
					end
					if not t[k][kk] then
						t[k][kk] = {}
					end
					t[k][kk][i] = kkk
					i = i + 1
				end
			else			
				for kkk, groups in pairs(player) do
					if groups then
						showClass = true
						
						if not t[k] then
							t[k] = {}
						end
						if not t[k][kk] then
							t[k][kk] = {}
						end
						t[k][kk][i] = kkk
						i = i + 1
					end
				end
			end
		end
	end
	
	if showClass then
		for k, buff in pairs(t) do
			if string.upper(name) == string.upper("Paladin") then
				local cat = tablet:AddCategory(
					'text', L[name] .. " - " .. L[k][1],
			        'columns', 1,
			        'child_textR', 1,
			        'child_textG', 1,
			        'child_textB', 0
				)
				for kk, players in pairs(buff) do
					cat:AddLine(
						'text', kk
					)
				end
			else
				local cat = tablet:AddCategory(
					'text', L[name] .. " - " .. L[k][1],
			        'columns', 2,
			        'child_textR', 1,
			        'child_textG', 1,
			        'child_textB', 0,
			        'child_textR2', 1,
			        'child_textG2', 1,
			        'child_textB2', 1
				)
				table.sort(buff)
				for kk, players in pairs(buff) do
					table.sort(players)
					
					cat:AddLine(
						'text', kk,
						'text2', table.concat(players, ", ")
					)
				end
			end
		end
	end
end

function kkbk:OnClick()
	if IsShiftKeyDown() then
		--for test only
		--self:addRemoveFromRaid("Reiko", true)

		local send_something = self:sendAssign("click", "all")
		
		if self.db.profile.channel == L:GetTranslation("Raid") then
			to = "RAID"
		elseif self.db.profile.channel == L:GetTranslation("Party") then
			to = "PARTY"
		else
			id, _ = GetChannelName(self.db.profile.channel_set)
			to = "CHANNEL"
		end
		if send_something and self.db.profile.answer_to_whisp then
			SendChatMessage(L:GetTranslation("SHOW_WHISP"), to, nil, id)
		end
	end
end

function kkbk:CHAT_MSG_WHISPER()
	if string.find(arg1, "kkbk") then
		if string.lower(arg1) == "kkbk" then
			self:sendAssign(arg2, "own")
		elseif string.find(arg1, "all") then
			self:sendAssign(arg2, "all")
		elseif string.find(arg1, string.lower(L:GetTranslation("Paladin"))) then
			self:sendAssign(arg2, "Paladin")
		elseif string.find(arg1, string.lower(L:GetTranslation("Priest"))) then
			self:sendAssign(arg2, "Priest")
		elseif string.find(arg1, string.lower(L:GetTranslation("Druid"))) then
			self:sendAssign(arg2, "Druid")
		elseif string.find(arg1, string.lower(L:GetTranslation("Mage"))) then
			self:sendAssign(arg2, "Mage")
		end
	end
end

function kkbk:sendAssign(from, what_i_send)
	local st = ""
	local send_something = false
	
	for class, classes in pairs(assign) do
		for buff, buffs in pairs(classes) do
			if string.upper(class) == string.upper("Paladin") then
				for player, do_it in pairs(buffs) do
					if (from == "click" or what_i_send == "all") or (what_i_send == "own" and from == player) 
						or what_i_send == "Paladin" then
						
						if do_it then 
							local strg = self.db.profile.paladin_buff_format
							strg = string.gsub(strg, "$u", player)
							strg = string.gsub(strg, "$b", self:getShortName(buff))
							
							self:sendAssignImpl(from, strg)
							
							send_something = true
						end
					end
				end	
			else
				for player, groups in pairs(buffs) do
					if (from == "click" or what_i_send == "all") or (what_i_send == "own" and from == player) 
						or (string.upper(what_i_send) == string.upper(class)) then
						
						local g = {}
						local o = 1
						for k, v in pairs(groups) do
							if v then
								g[o] = k
								o = o + 1
							end
						end
						
						if table.getn(g) > 0 then
							local strg = self.db.profile.buff_format
							strg = string.gsub(strg, "$u", player)
							strg = string.gsub(strg, "$b", self:getShortName(buff))
							strg = string.gsub(strg, "$g", table.concat(g, self.db.profile.separator))
							
							self:sendAssignImpl(from, strg)
							
							send_something = true
						end
					end
				end
			end
		end
	end
	
	if not send_something and from ~= "click" then
		if what_i_send == "own" then
			self:sendAssignImpl(from, L:GetTranslation("You don't have anything to buff"))
		elseif what_i_send == L:GetTranslation("Paladin") or what_i_send == L:GetTranslation("Priest")
			or what_i_send == L:GetTranslation("Mage") or what_i_send == L:GetTranslation("Druid") then
			self:sendAssignImpl(from, L:GetTranslation("Nothing to buff for") .. " " .. what_i_send)
		end
	end
	return send_something
end

function kkbk:sendAssignImpl(from, line)
	local to, id
	
	if from == "click" then
		if self.db.profile.channel == L:GetTranslation("Raid") then
			to = "RAID"
		elseif self.db.profile.channel == L:GetTranslation("Party") then
			to = "PARTY"
		else
			id, _ = GetChannelName(self.db.profile.channel_set)
			to = "CHANNEL"
		end
	else
		to = "WHISPER"
		id = from
	end
	
	--self:Debug(line, to, id)
	SendChatMessage(line, to, nil, id)
end

function kkbk:getShortName(buff)
	local l = { 
		"FORTITUDE", "SPIRIT", "SHADOW",
		"MARKOFWILD", "THORNS",
		"INTELLECT", "AMPLIFY", "DAMPEN",
		"KING", "WISDOM", "MIGHT", "LIGHT", "SALVATION", "SANCTUARY",
	}
	
	for _, v in pairs(l) do
		if buff == v then
			return L[v][2]
		end
	end
	
	-- we don't find it (zomg oO)
	return buff
end

function kkbk:sendAddon(class, buff, name, party, value, palouf)
	local partyOrNil, paloufOrNil, valueText
	if party then partyOrNil = party else partyOrNil = "nil" end
	if palouf then paloufOrNil = palouf else paloufOrNil = "nil" end
	if value then valueText = "true" else valueText = "false" end
	
	local line = class .. "#" .. buff .."#" .. name .. "#" ..partyOrNil .. "#" .. valueText .. "#" .. paloufOrNil
	self:Debug("KKBK" .. self.version, line, "RAID")
	
	SendAddonMessage("KKBK" .. self.version, line, "RAID")	
end

local request_string = "REQUEST"
local request_all = "ALL"
local request_join_raid = "JOIN_RAID"
local request_version = "VERSION"
local answer_string = "ANSWER"

function kkbk:CHAT_MSG_ADDON()
	--[[ arg1 prefix 
		arg2 message 
		arg3 distribution type ("PARTY","RAID","GUILD" or "BATTLEGROUND") 
		arg4 sender]]
	--self:Debug("receive", arg1, arg2, arg3, arg4)
	if arg1 == "KKBK" .. self.version then
		-- don't handle own send... 
		if arg4 ~= UnitName("player") or really_debug then
			local arr = self:split(arg2, "#")
			
			if table.getn(arr) == 6 then
				local class = arr[1]
				local buff = arr[2]
				local name = arr[3]
				local party = tonumber(arr[4])
				local value = arr[5]
				if value == "true" then value = true else value = false end
				local palouf = arr[6]
				
				if palouf == "Paladin" then
					self:Debug("need to set", buff, "to", class, "and user", name, "with value", value)
					
					if assign[class] then
						if assign[class][buff] then
							if assign[class][buff][name] ~= nil then
								assign[class][buff][name] = value
							end
						end
					end
				else		
					--self:printTable()
					self:Debug("need to set", buff, "to", class, "and user", name, "in group", party, "with value", value)
					
					if assign[class] then
						if assign[class][buff] then
							if assign[class][buff][name] then
								if assign[class][buff][name][party] ~= nil then
									assign[class][buff][name][party] = value
								end
							end
						end
					end
				end
			else --if table.getn(arr) == 3 then
				local request = arr[1]
				local type = arr[2]
				local arg = arr[3]
				self:Debug("receive", request, "type", type, "args", arg)

				if request == request_string then
					if type == request_join_raid then
						if arg == request_all then
							if self.db.profile.show_only_my_class then
								self:sendAssignRequest(self:getOwnClass())
							else
								for _, v in pairs(classes) do
									self:sendAssignRequest(v)
								end
							end
						elseif arg == self:getOwnClass() then
							self:sendAssignRequest(arg)
						end
					elseif type == request_version then
						local line = answer_string .. "#" .. request_version .. "#" .. self.version
						SendAddonMessage("KKBK" .. self.version, line, "RAID")
					end
				elseif request == answer_string then
					if type == request_version then
					    self:Debug(arg4, "has version", arg)
					end
				end
			end
		end
	end
end

function kkbk:requestVersion()
	self:Debug("KKBK" .. self.version, "REQUEST#VERSION#", "RAID")
	SendAddonMessage("KKBK" .. self.version, "REQUEST#VERSION#", "RAID")
end

function kkbk:sendAssignRequest(arg)
	if assign[arg] then
		for buff, player in pairs(assign[arg]) do
			for player, groups in pairs(player) do
				if arg == "Paladin" then
					if groups then
						local line = arg .. "#" .. buff .."#" .. player .. "#nil#true#Paladin"
						self:Debug("KKBK" .. self.version, line, "RAID")
						SendAddonMessage("KKBK" .. self.version, line, "RAID")
					end
				else
					for k, v in pairs(groups) do
						if v then
							local line = arg .. "#" .. buff .."#" .. player .. "#" ..k .. "#true#nil"
							self:Debug("KKBK" .. self.version, line, "RAID")
							SendAddonMessage("KKBK" .. self.version, line, "RAID")
						end
					end
				end
			end
		end
	end
end

function kkbk:requestAssignments()
	self:Debug("raid joined, ask assign")
	
	local line
	if self.db.profile.show_only_my_class then
		line = request_string .. "#" .. request_join_raid .. "#" .. self:getOwnClass()
	else
	    line = request_string .. "#" .. request_join_raid .. "#" .. request_all
	end
	self:Debug("KKBK" .. self.version, line, "RAID")

	SendAddonMessage("KKBK" .. self.version, line, "RAID")
end

function kkbk:split(msg, char)
	local arr = { };
	while (string.find(msg, char) ) do
		local iStart, iEnd = string.find(msg, char);
		tinsert(arr, strsub(msg, 1, iStart-1));
		msg = strsub(msg, iEnd+1, strlen(msg));
	end
	if ( strlen(msg) > 0 ) then
		tinsert(arr, msg);
	end
	return arr;
end
