--[[ Bootstrapping ]]

mEnemyCastbar = AceLibrary("AceAddon-2.0"):new("AceEvent-2.0", "CandyBar-2.0", "AceConsole-2.0", "AceDB-2.0");

mecb = mEnemyCastbar;
local isLive;
_,isLive = GetBuildInfo();
isLive = isLive == "1.12.2";

local print = function(s) if (mecb.db.profile.logevents) then ChatFrame1:AddMessage(tostring(s), 1, 0.5, 0) end end;

mecb.units = {
	target = true, 
	party1 = true, 
	party2 = true, 
	party3 = true, 
	party4 = true, 
	party5 = true
};

mecb.validUnits = {
	"party1", "party2", "party3", "party4", "party5", 
	"raid1" , "raid2" , "raid3" , "raid4" , "raid5" , 
	"raid6" , "raid7" , "raid8" , "raid9" , "raid10", 
	"raid11", "raid12", "raid13", "raid14", "raid15", 
	"raid16", "raid17", "raid18", "raid19", "raid20", 
	"raid21", "raid22", "raid23", "raid24", "raid25", 
	"raid26", "raid27", "raid28", "raid29", "raid30", 
	"raid31", "raid32", "raid33", "raid34", "raid35", 
	"raid36", "raid37", "raid38", "raid39", "raid40", 
	"player", "target", "pet"   , "focus" ,
}

mecb.guardUnits = { "player" }

--[[ AceDB ]]

mecb:RegisterDB("mecbDB");
mecb:RegisterDefaults("profile", {
	units = {
		target = true, 
		party1 = true, 
		party2 = true, 
		party3 = true, 
		party4 = true, 
		party5 = true
	},
	logevents = false,
	events = {},
});

--[[ Console Handlers ]]

function mecb:GetUnits()
	local t = {};
	for k,_ in pairs(self.db.profile.units) do
		table.insert(t, k);
	end
	
	table.sort(t);
	return t;
end

function mecb:AddUnit(id)
	self.db.profile.units[id] = true;
end

function mecb:RemoveUnit(id)
	self.db.profile.units[id] = nil;
end

function mecb:ListUnits()
	local t = self:GetUnits();
	self:Print("Watched units are: " .. table.concat(t, ", "));
end

--[[
List of events concerning spell casting

* UNIT_SPELLCAST_START, unit
* UNIT_SPELLCAST_STOP, unit
* UNIT_SPELLCAST_FAILED, unit
* UNIT_SPELLCAST_INTERRUPTED, unit
* UNIT_SPELLCAST_DELAYED, unit
* UNIT_SPELLCAST_SUCCEEDED, "player", spell, rank
* UNIT_SPELLCAST_CHANNEL_START, unit
* UNIT_SPELLCAST_CHANNEL_UPDATE, unit
* UNIT_SPELLCAST_CHANNEL_STOP, unit

List of events we're registering
* UNIT_SPELLCAST_START, unit
  Normal spell cast

* UNIT_SPELLCAST_CHANNEL_START, unit
  Channeled spell cast

* UNIT_SPELLCAST_STOP, unit
* UNIT_SPELLCAST_CHANNEL_STOP, unit
  Spell finished

* UNIT_SPELLCAST_DELAYED, unit
* UNIT_SPELLCAST_CHANNEL_UPDATE, unit
  Time changed

* PLAYER_TARGET_CHANGED
  Target changed, update casting bar
]]

mecb.SpellStatus = {};

function mecb:OnInitialize()
	mecb:RegisterChatCommand({"/mecb"}, {
		type = "group",
		args = {
			addunit = {
				name = "addunit",
				desc = "Adds a unit to the list to watch",
				type = "text",
				get = false,
				set = "AddUnit",
				validate = mecb.validUnits,
				usage = "[unitid]",
			},
			removeunit = {
				name = "removeunit",
				desc = "Remove a unit from the list to watch",
				type = "text",
				get = false,
				set = "RemoveUnit";
				usage = "[unitid]",
			},
			listunits = {
				name = "listunits",
				desc = "Lists all watched units",
				type = "execute",
				func = "ListUnits",
			},
			logevents = {
				name = "logevents",
				desc = "Log all casting events",
				type = "toggle",
				get = function () return self.db.profile.logevents end,
				set = function (v) self.db.profile.logevents = v end,
			},
		}
	})
end

function mecb:OnEnable()
	-- AceEvent: Registration of events
	self:RegisterEvent("UNIT_SPELLCAST_START", "Dispatch");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_START", "Dispatch");
	
	self:RegisterEvent("UNIT_SPELLCAST_STOP", "DispatchStop");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_STOP", "DispatchStop");
	
	self:RegisterEvent("UNIT_SPELLCAST_DELAYED", "Dispatch");
	self:RegisterEvent("UNIT_SPELLCAST_CHANNEL_UPDATE", "Dispatch");
	
	self:RegisterEvent("UNIT_SPELLCAST_FAILED", "DispatchStop");
	self:RegisterEvent("UNIT_SPELLCAST_INTERRUPTED", "DispatchStop");
	
	self:RegisterEvent("PLAYER_TARGET_CHANGED", "OnTargetChanged");
	
	self:ScheduleRepeatingEvent("Guardian", self.Guardian, 0.2, self);
end

-- CandyBar hates me, it never does what I mean
function mecb:ezcandy(id, endTime, time, text, icon, reversed, c1, c2, c3, c4, c5, c6, c8, c9, c10)
    --self:RegisterCandyBarGroup(group)
    --self:SetCandyBarGroupGrowth(group, true)
	local _, _, pnum = string.find(id, "party(%d)");
	if (pnum) then id2 = "aUFgrouppartyUnitButton" .. pnum; --[[ print(id2 .. ", " .. tostring(aUF.units[id2]));]] end;
	
	local aUFunit = aUF.units["aUF" .. id] or aUF.units[id] or aUF.units[id2];
	if (not aUFunit) then return end;

	self:RegisterCandyBar(id, time, text, icon, c1, c2, c3, c4, c5, c6, c8, c9, c10)
    --self:RegisterCandyBarWithGroup(id, group)

    self:SetCandyBarFade(id, .5)
	self:SetCandyBarReversed(id, reversed)

	self:SetCandyBarWidth(id, aUFunit.database.Width*1.15)
	self:SetCandyBarScale(id, aUFunit.frame:GetEffectiveScale()/1.15)
	self:SetCandyBarPoint(id, "BOTTOM", aUFunit.frame, "TOP", 0, 0)
	self:SetCandyBarTexture(id, aUF.Bars[aUF.db.profile.BarStyle]);

	-- self:Print(aUF.units.target.database.Width);
    self:StartCandyBar(id, false)
	
	-- Okay. Now we have to sync it to the clock.
	self:SetCandyBarTimeLeft(id, endTime/1000 - GetTime())
end

function mecb:OnDisable()
	-- AceEvent: Unregister all events
	self:UnregisterAllEvents()
	self:CancelScheduledEvent("Guardian");
end

--[[

function mecb:OnSpellStart(unit)
	mecb:LogEvent(unit);
	if (not self.db.profile.units[unit]) then return end;
	local name, nameSubtext, text, texture, startTime, endTime, isTradeSkill = UnitCastingInfo(unit);
	local duration = (endTime - startTime) / 1000;
	self:ezcandy(unit, duration, text, texture, true, "yellow", "green")
	local left = (endTime/1000 - GetTime());
	self:SetCandyBarTimeLeft(unit, left);
	
	mecb.SpellStatus[unit.."start"] = startTime;
	--print(format("SpellStart [%s] at %.1f", unit, startTime/1000));
end

function mecb:OnChannelStart(unit)
	mecb:LogEvent(unit);
	if (not self.db.profile.units[unit]) then return end;
	local name, nameSubtext, text, texture, startTime, endTime, isTradeSkill = UnitChannelInfo(unit);
	local duration = (endTime - startTime) / 1000;
	self:ezcandy(unit, duration, text, texture, false, "yellow", "cyan")
	--self:SetCandyBarTimeLeft(unit, (endTime - GetTime()) / 1000);
	local left = (endTime/1000 - GetTime());
	--self:Print(startTime, endTime);
	--self:Print("Time left: %.1f", left);
	self:SetCandyBarTimeLeft(unit, left);

	mecb.SpellStatus[unit.."start"] = startTime;
	--print(format("ChannelStart [%s] at %.1f with duration %.1f", unit, startTime/1000, duration));
end

function mecb:OnSpellStop(unit)
	mecb:LogEvent(unit);
	if (not self.db.profile.units[unit]) then return end;
	if (UnitCastingInfo(unit)) then return end;
	
	self:StopCandyBar(unit);
end

function mecb:OnSpellDelayed(unit)
	mecb:LogEvent(unit);
	if (not self.db.profile.units[unit]) then return end;

	local dbstart = mecb.SpellStatus[unit.."start"];
	
	local _, _, _, _, startTime, endTime = UnitCastingInfo(unit);
	local duration = (endTime - dbstart) / 1000;
	self:StopCandyBar(unit);
	self:SetCandyBarTime(unit, duration);
	self:StartCandyBar(unit);

	local elapsed = GetTime() - dbstart/1000;
	local left = duration - elapsed;
	self:SetCandyBarTimeLeft(unit, left);
	
	--print(format("SpellDelayed [%s], setting duration to %.1f / elapsed: %.1f / left: %.1f", unit, duration, elapsed, left));
end

function mecb:OnChannelUpdate(unit)
	mecb:LogEvent(unit);
	if (not self.db.profile.units[unit]) then return end;

	local dbstart = mecb.SpellStatus[unit.."start"];

	local _, _, _, _, startTime, endTime = UnitChannelInfo(unit);
	local duration = (endTime - startTime) / 1000;
	--self:StopCandyBar(unit);
	--self:SetCandyBarTime(unit, duration);
	--self:StartCandyBar(unit);
	
	local elapsed = GetTime() - dbstart/1000;
	local left = duration - elapsed;
	
	self:SetCandyBarTimeLeft(unit, left);

	--print(format("ChannelUpdate [%s], setting duration to %.1f / elapsed: %.1f / left: %.1f", unit, duration, elapsed, left));
end

function mecb:OnSpellFailed(unit)
	mecb:LogEvent(unit);
	if (not self.db.profile.units[unit]) then return end;
	if (UnitCastingInfo(unit)) then return end;

	self:SetCandyBarColor(unit, "red");
	self:SetCandyBarFade(unit, 1);
	self:StopCandyBar(unit);
end

]]

function mecb:OnTargetChanged()
	local nameChannel  = UnitChannelInfo("target");
	local nameSpell  = UnitCastingInfo("target");
	if ( nameChannel ) then
		mecb.doChannelStart("target", UnitChannelInfo("target"));
	elseif ( nameSpell ) then
		mecb.doCastStart("target", UnitCastingInfo("target"));
	else
		mecb:StopCandyBar("target");
	end
end

--[[

function mecb:LogEvent(unit)
	local t;
	
	if (UnitChannelInfo(unit)) then
		t = {event, unit, UnitChannelInfo(unit)};
	else
		t = {event, unit, UnitCastingInfo(unit)};
	end
	
    for k,v in ipairs(t) do t[k] = tostring(v) end;
    local s = table.concat(t, ", ") .. "\n";
    TinyPadEditBox:SetText(TinyPadEditBox:GetText() .. s);
	
	--tinsert(self.db.profile.events, s);
	--print(s);
end

]]

local handlers = mecb;
local unitinfo = {};

function handlers.doCastStart(unit, name, nameSubtext, text, texture, startTime, endTime, isTradeSkill, event)
	-- print ("*"..event.."*", unit, name, texture, startTime, endTime);
	
	unitinfo[unit] = unitinfo[unit] or {};
	
	unitinfo[unit].casting = true;
	unitinfo[unit].startTime = startTime;
	unitinfo[unit].endTime = endTime;
	local duration = endTime - startTime;
	unitinfo[unit].duration = duration;
	unitinfo[unit].delay = 0;
	unitinfo[unit].name = name;
	
	print(string.format("- CastStart @ %.1f: %s with duration %.1f, end at %.1f", startTime/1000, name, duration/1000, endTime/1000));
	
	mecb:ezcandy(unit, endTime, duration/1000, name, texture, true, "yellow", "green");
end

function handlers.doChannelStart(unit, name, nameSubtext, text, texture, startTime, endTime, isTradeSkill)
	unitinfo[unit] = unitinfo[unit] or {};
	
	unitinfo[unit].channeling = true;
	unitinfo[unit].startTime = startTime;
	unitinfo[unit].endTime = endTime;
	local duration = endTime - startTime;
	unitinfo[unit].duration = duration;
	unitinfo[unit].delay = 0;
	unitinfo[unit].name = name;
	
	print(string.format("- ChannelStart @ %.1f: %s with duration %.1f, end at %.1f", startTime/1000, name, duration/1000, endTime/1000));
	
	mecb:ezcandy(unit, endTime, duration/1000, name, texture, false, "yellow", "cyan");
end

function handlers.doCastStop(unit)
	unitinfo[unit] = unitinfo[unit] or {};
	if (unitinfo[unit].casting) then
		unitinfo[unit].casting = false;
		print("* CastStop");
	elseif (unitinfo[unit].channeling) then
		print("* CastStop (Channeling)");
		return;
	else
		print("* CastStop (S)");
	end
	
	mecb:StopCandyBar(unit);
end

function handlers.doChannelStop(unit)
	unitinfo[unit] = unitinfo[unit] or {};
	if (unitinfo[unit].channeling) then
		unitinfo[unit].channeling = false;
		print("* ChannelStop");
	else
		print("* ChannelStop (S)");
	end
	
	mecb:StopCandyBar(unit);
end

function handlers.doChannelUpdate(unit, name, nameSubtext, text, texture, startTime, endTime, isTradeSkill)
	if (not unitinfo[unit] or not unitinfo[unit].channeling) then
		print("! ChannelDelay while not casting");
		return;
	end
	
	local delay = unitinfo[unit].startTime - startTime;
	local duration = endTime - startTime - delay;
	unitinfo[unit].duration = duration;
	unitinfo[unit].delay = delay;
	
	print(string.format("! ChannelUpdate @ %.1f: %s with duration %.1f, end at %.1f, delay is %.1f", startTime/1000, name, duration/1000, endTime/1000, delay/1000));
	
	mecb:SetCandyBarTimeLeft(unit, endTime/1000 - GetTime());
	mecb:SetCandyBarText(unit, unitinfo[unit].name .. " (|cffff4444-" .. string.format("%.1f", delay/1000) .. "|r)");
end

function handlers.doCastFailed(unit)
	unitinfo[unit] = unitinfo[unit] or {};
	if (unitinfo[unit].casting) then
		unitinfo[unit].casting = false;
		print("- CastFailed (C)");
	else
		print("- CastFailed (S)");
	end
	
	mecb:SetCandyBarColor(unit, "red");
	mecb:SetCandyBarFade(unit, 1);
	mecb:StopCandyBar(unit);
end


function handlers.doCastDelayed(unit, name, nameSubtext, text, texture, startTime, endTime, isTradeSkill)
	-- print ("*"..event.."*", unit, name, texture, startTime, endTime);
	
	if (not unitinfo[unit] or not unitinfo[unit].casting) then
		print("! CastDelayed while not casting");
		return;
	end
	
	local delay = startTime - unitinfo[unit].startTime;
	unitinfo[unit].endTime = endTime;
	local duration = endTime - startTime + delay;
	unitinfo[unit].duration = duration;
	unitinfo[unit].delay = delay;
	
	print(string.format("! CastDelayed @ %.1f: %s with duration %.1f, end at %.1f, delay is %.1f", startTime/1000, name, duration/1000, endTime/1000, delay/1000));
	
	mecb:SetCandyBarTime(unit, duration/1000);
	mecb:SetCandyBarText(unit, unitinfo[unit].name .. " (|cffff4444" .. string.format("%.1f", delay/1000) .. "|r)");
end

local dispatcher = {
	UNIT_SPELLCAST_START = handlers.doCastStart;
	UNIT_SPELLCAST_CHANNEL_START = handlers.doChannelStart;
	UNIT_SPELLCAST_STOP = handlers.doCastStop;
	UNIT_SPELLCAST_CHANNEL_STOP = handlers.doChannelStop;
	UNIT_SPELLCAST_FAILED = handlers.doCastFailed;
	UNIT_SPELLCAST_INTERRUPTED = handlers.doCastFailed;
	UNIT_SPELLCAST_DELAYED = handlers.doCastDelayed;
	UNIT_SPELLCAST_CHANNEL_UPDATE = handlers.doChannelUpdate;
}

function mecb:Dispatch(unit)
	if (event and unit and self.db.profile.units[unit] and dispatcher[event]) then
		if (UnitCastingInfo(unit)) then
			dispatcher[event](unit, UnitCastingInfo(unit));
		else
			dispatcher[event](unit, UnitChannelInfo(unit));
		end
	end
end

function mecb:DispatchStop(unit)
	if (event and unit and self.db.profile.units[unit] and dispatcher[event]) then
		dispatcher[event](unit);
	end
end

local count = 0;

function mecb:Guardian()
	for k,v in ipairs(mecb.guardUnits) do
		local info = unitinfo[v];
		if (info) then 
			if (info.casting) then
				local _, _, _, _, _, endTime = UnitCastingInfo(v);
				-- print(string.format("Guardian (%s): %.1f %.1f", v, info.endTime, endTime or 0));
				if (endTime and info.endTime ~= endTime) then
					print("# Guardian - invoking CastDelay");
					mecb.doCastDelayed(v, UnitCastingInfo(v));
				end
			elseif (info.channeling) then
				local _, _, _, _, _, endTime = UnitChannelInfo(v);
				if (endTime and info.endTime ~= endTime) then
					print("# Guardian - invoking ChannelUpdate");
					mecb.doChannelUpdate(v, UnitChannelInfo(v));
				end
			end
		end
	end
end

if (not UnitChannelInfo) then
	function UnitChannelInfo(unit)
		if (not UnitExists(unit)) then return end
		return "name", "nameSubText", "Arcane Missiles", "Interface\\Icons\\Spell_Arcane_Blink.blp", GetTime(), GetTime() + 5000, false;
	end
end

if (not UnitCastingInfo) then
	function UnitCastingInfo(unit) 
		if (not UnitExists(unit)) then return end
		return "name", "nameSubText", "Fireball", "Interface\\Icons\\Spell_Fire_Fireball.blp", GetTime(), GetTime() + 3000, false;
	end
end

